@@ -239,6 +239,11 @@ bool cb_window_expose (GtkWidget *widget, GdkEventExpose *event, Application * a
     return false;
 }
 
+static void windowTopMost (GObject * self, GParamSpec *, Application * app) {
+    if (app->fullscreen && hildon_window_get_is_topmost (app->window))
+        app->normalScreen ();
+}
+
 void Application::setPlaying (bool p) {
     playing = p;
     if (playing) {
@@ -389,6 +394,8 @@ static void create_menu (Application * app) {
     app->menu_item_media_server = gtk_radio_menu_item_new_with_label (NULL, "Osso Media Server");
     app->menu_item_mplayer = gtk_radio_menu_item_new_with_label_from_widget(
             GTK_RADIO_MENU_ITEM (app->menu_item_media_server), "MPlayer");
+    app->menu_item_npp = gtk_radio_menu_item_new_with_label_from_widget(
+            GTK_RADIO_MENU_ITEM (app->menu_item_media_server), "Flash");
     
     /* Add menu items to right menus */
     gtk_menu_append (main_menu, item_new);
@@ -407,6 +414,7 @@ static void create_menu (Application * app) {
     gtk_menu_append (menu_playlist, app->menu_item_playlist_edit);
     gtk_menu_append (menu_players, app->menu_item_media_server);
     gtk_menu_append (menu_players, app->menu_item_mplayer);
+    gtk_menu_append (menu_players, app->menu_item_npp);
     gtk_menu_append( main_menu, item_close );
 
     /* Add others submenu to the "Others" item */
@@ -444,6 +452,10 @@ static void create_menu (Application * app) {
         GTK_SIGNAL_FUNC (togglePlaylistEdit), app);
     g_signal_connect (G_OBJECT (app->menu_item_media_server), "toggled",
         GTK_SIGNAL_FUNC (toggleMediaServer), app);
+    g_signal_connect (G_OBJECT (app->menu_item_mplayer), "toggled",
+        GTK_SIGNAL_FUNC (toggleMediaServer), app);
+    g_signal_connect (G_OBJECT (app->menu_item_npp), "toggled",
+        GTK_SIGNAL_FUNC (toggleMediaServer), app);
 
     g_signal_connect (G_OBJECT (main_menu), "show",
         GTK_SIGNAL_FUNC (mainMenuShow), app);
@@ -594,6 +606,8 @@ int main (int argc, char* argv[]) {
         GTK_SIGNAL_FUNC (cbViewAreaFullscreenUnmapped), app);
     g_signal_connect (G_OBJECT (app->window), "expose-event",
         GTK_SIGNAL_FUNC (cb_window_expose), app);
+    g_signal_connect (G_OBJECT (app->window), "notify::is-topmost",
+        GTK_SIGNAL_FUNC (windowTopMost), app);
     gtk_paned_add1 (app->vsplit_view, app->view_area);
     app->infopanel_view = gtk_text_view_new ();
     app->info_viewport = gtk_scrolled_window_new (0L, 0l);
