@@ -169,6 +169,7 @@ public:
     virtual void postpone (bool b);
     virtual void clipStart ();
     virtual void clipStop ();
+    virtual SRect intrinsicBounds ();
     CalculatedSizer sizes;
     PostponePtr postpone_lock;
     Fit fit;
@@ -202,6 +203,7 @@ public:
     virtual void postpone (bool b);
     virtual void clipStart ();
     virtual void clipStop ();
+    virtual SRect intrinsicBounds ();
     CachedImage cached_img;
     bool have_frame;
 protected:
@@ -399,6 +401,7 @@ public:
      * repaints region, calls scheduleRepaint(x,y,w,h) on view
      */
     void repaint ();
+    void repaint (const SRect & rect);
     /**
      * calculate the relative x,y,w,h on the child region elements
      * given this element's w and h value
