@@ -191,8 +191,8 @@ void Control::play () {
             m_app->playlist_data->document->defer ();   // pause
     } else {
         debugLog () << "play" << endl;
-        gtk_paned_set_position (m_app->vsplit_view, VSPLIT);
-        gtk_container_resize_children (GTK_CONTAINER (m_app->vsplit_view));
+        //gtk_paned_set_position (m_app->vsplit_view, VSPLIT);
+        //gtk_container_resize_children (GTK_CONTAINER (m_app->vsplit_view));
         if (!applyBackRequest () &&
                 m_app->playlist_data->document) {
             playCurrent ();
@@ -214,12 +214,13 @@ void Control::stop () {
     if (m_app->playlist_data->document &&
             m_app->playlist_data->document->state > Node::state_init) {
         m_app->playlist_data->document->reset ();
-        gtk_paned_set_position (m_app->vsplit_view, 40);
+        //gtk_paned_set_position (m_app->vsplit_view, 40);
         setCurrent (0L);
     }
     m_app->viewarea->setAspect (0.0);
     m_process->quit ();
     gdk_window_clear (m_app->viewarea->videoWindow ());
+    gdk_window_clear (m_app->viewarea->viewWidget ()->window);
     gtk_widget_queue_draw (m_app->infopanel_view);
     m_app->normalScreen ();
     setLength (0);
@@ -322,8 +323,12 @@ bool Control::resolveURL (NodePtr m) {
         return true;
     URL url (mrl->absolutePath ());
     String mimestr = mrl->mimetype;
+    if (mimestr == "application/x-shockwave-flash" ||
+            mimestr == "application/futuresplash")
+        return true; // FIXME
     if (mimestr.isEmpty ()) {
-        //mimestr = MimeType::findByURL (url);
+        if (url.isLocalFile ())
+            mimestr = MimeType::findByURL (url);
         if (!mimestr.isEmpty ())
             mrl->mimetype = mimestr;
     }
@@ -626,14 +631,22 @@ void Control::updateViewAreaSizes () {
     Rect video_rect = m_app->viewarea->videoGeometry ();
     int x = video_rect.x ();
     int y = video_rect.y ();
-    for (GdkWindow * win = gdk_window_get_parent (
-                m_app->viewarea->videoWindow ());
-            win;
-            win = gdk_window_get_parent (win)) {
-        int x1, y1;
-        gdk_window_get_position (win, &x1, &y1);
-        x += x1;
-        y += y1;
+#ifdef __ARMEL__
+    if (process () == m_app->npp) {
+        m_process->setVideoWindow (x, y,
+                video_rect.width (), video_rect.height ());
+    } else
+#endif
+    {
+        for (GdkWindow * win = gdk_window_get_parent (
+                    m_app->viewarea->videoWindow ());
+                win;
+                win = gdk_window_get_parent (win)) {
+            int x1, y1;
+            gdk_window_get_position (win, &x1, &y1);
+            x += x1;
+            y += y1;
+        }
     }
     debugLog () << "updateViewAreaSizes " << x << ", " << y << " " << video_rect.width () << "x" << video_rect.height () << endl;
     m_process->setVideoWindow (x, y,
@@ -850,6 +863,9 @@ void Application::initControl () {
     viewarea = new ViewArea;
     media_server = new OssoMediaServer (control, osso_context, has_xv);
     mplayer = new MPlayer (control, has_xv);
+#ifdef __ARMEL__
+    npp = new NpPlayer (control);
+#endif
     if (!strcmp (OssoMediaServer::myname, (const char *)
             config()->readEntry (GCONF_KEY_PLAYER, OssoMediaServer::myname))) {
         control->setProcess (media_server);
@@ -903,16 +919,12 @@ void Application::setInfoContent (const char * utf8, bool plaintext) {
     if (!plaintext && utf8[0]) {
         if (!m_config)
             m_config = new Config; // force GConf init
-        if (isplit_view == info_viewport->parent)
-            gtk_container_remove (GTK_CONTAINER (isplit_view), info_viewport);
-        if (isplit_view != html_viewport->parent)
-            gtk_container_add (GTK_CONTAINER (isplit_view), html_viewport);
+        gtk_widget_hide (info_viewport);
+        gtk_widget_show (html_viewport);
         gtk_html_load_from_string (GTK_HTML (html_infopanel_view), utf8, -1);
     } else {
-        if (isplit_view != info_viewport->parent) {
-            gtk_container_remove (GTK_CONTAINER (isplit_view), html_viewport);
-            gtk_container_add (GTK_CONTAINER (isplit_view), info_viewport);
-        }
+        gtk_widget_hide (html_viewport);
+        gtk_widget_show (info_viewport);
         gtk_text_buffer_set_text (gtk_text_view_get_buffer
                 (GTK_TEXT_VIEW (infopanel_view)), utf8, -1);
     }
@@ -1042,13 +1054,22 @@ void progressValueChanged(GtkWidget *w, Application * app) {
     if (!app->control->in_progress_update) {
         int p = hildon_seekbar_get_position(HILDON_SEEKBAR (app->progress_bar));
         app->control->process ()->seek (p, true);
-        debugLog () << "progressBarClicked " << p << endl;
+        //debugLog () << "progressBarClicked " << p << endl;
     }
 }
 
 void toggleMediaServer (GtkCheckMenuItem *radio, Application * app) {
-    Process * p =  gtk_check_menu_item_get_active (radio) 
-        ? app->media_server : app->mplayer;
+    if (!gtk_check_menu_item_get_active (radio))
+        return;
+    Process * p;
+    if (radio == GTK_CHECK_MENU_ITEM (app->menu_item_media_server))
+        p = app->media_server;
+#ifdef __ARMEL__
+    else if (radio == GTK_CHECK_MENU_ITEM (app->menu_item_npp))
+        p = app->npp;
+#endif
+    else
+        p = app->mplayer;
     app->control->setProcess (p);
     app->config()->writeEntry (GCONF_KEY_PLAYER, String (p->name ()));
 }
