@@ -28,6 +28,10 @@
 #include <dbus/dbus.h>
 
 #include "kmplayerprocess.h"
+#ifdef __ARMEL__
+# include "npplayer.h"
+# include "kmplayer_lists.h"
+#endif
 
 using namespace KMPlayer;
 
@@ -44,14 +48,6 @@ Process::~Process () {
         g_source_remove (start_timer);
 }
 
-void Process::init () {
-}
-
-void Process::initProcess (unsigned int viewer) {
-    m_viewer = viewer;
-    m_url.truncate (0);
-}
-
 //WId Process::widget () {
 //    return 0;
 //}
@@ -173,22 +169,22 @@ unsigned int Process::viewer () const {
 
 //-----------------------------------------------------------------------------
 
-void setupMPlayerFunc (gpointer data) {
+static void setupMPlayerFunc (gpointer data) {
     static_cast <MPlayer *> (data)->setState (Process::Playing); //FIXME
 }
 
-void watchMPlayerFunc (GPid pid, gint status, gpointer data) {
+static void watchMPlayerFunc (GPid pid, gint status, gpointer data) {
     //debugLog () << "watchMPlayerFunc " << endl;
     g_spawn_close_pid (pid);
     static_cast <MPlayer *> (data)->processExited (pid);
 }
 
-gboolean watchMPlayerStdin (GIOChannel *, GIOCondition, gpointer data) {
+static gboolean watchMPlayerStdin (GIOChannel *, GIOCondition, gpointer data) {
     static_cast <MPlayer *> (data)->writeEvent ();
     return true;
 }
 
-gboolean watchMPlayerStdout (GIOChannel *src, GIOCondition, gpointer data) {
+static gboolean watchMPlayerStdout (GIOChannel *src, GIOCondition, gpointer data) {
     static_cast <MPlayer *> (data)->readEvent (src);
     return true;
 }
@@ -203,9 +199,6 @@ MPlayer::MPlayer (ProcessNotify * ctrl, bool xv)
 MPlayer::~MPlayer () {
 }
 
-void MPlayer::initProcess (unsigned int) {
-}
-
 bool MPlayer::play (NodePtr node) {
     debugLog () << "MPlayer::play" << endl;
     if (playing ())
@@ -628,7 +621,7 @@ static DBusHandlerResult filter_func (DBusConnection * connection,
                     dbus_message_iter_get_basic (&iter, &state);
                     printf ("state_changed %s\n", state);
                     if (!strcmp (state, "stopped")) {
-                        if (!process->play_pending)
+                        if (!process->play_pending && !process->retry ())
                             process->processStopped ();
                     } else if (!strcmp (state, "playing"))
                         process->processPlaying ();
@@ -705,8 +698,8 @@ static DBusHandlerResult filter_func (DBusConnection * connection,
     } else if (message_type == DBUS_MESSAGE_TYPE_SIGNAL &&
             !strcmp (dbus_message_get_interface (message),
                 "com.nokia.osso_media_server.video.error")) {
-        warningLog() << "Unsupported format" << endl;
         const char * method = dbus_message_get_member (message);
+        warningLog() << "Unsupported format " << method << endl;
         if (!strcmp (method, "unsupported_type")) {
             if (!process->retry ())
                 process->processStopped ();
@@ -848,10 +841,6 @@ KDE_NO_CDTOR_EXPORT OssoMediaServer::~OssoMediaServer () {
         setVideoWindow (200, 60, 560, 380); // something sane
 }
 
-KMPLAYER_NO_EXPORT void OssoMediaServer::initProcess (unsigned int viewer) {
-    Process::initProcess (viewer);
-}
-
 KMPLAYER_NO_EXPORT bool OssoMediaServer::stop () {
     if (play_pending) {
         dbus_pending_call_cancel (play_pending);
@@ -911,9 +900,13 @@ KMPLAYER_NO_EXPORT bool OssoMediaServer::ready (unsigned int viewer) {
 }
 
 KMPLAYER_NO_EXPORT bool OssoMediaServer::retry () {
-    if (m_mrl && !m_mrl->mrl ()->audio_only) {
+    if (!current_service || !strcmp (current_service, "com.nokia.osso_media_server.music")) {
+        return false;
+    } else if (retry_timer) {
+        return true;
+    } else if (m_mrl && !m_mrl->mrl ()->audio_only) {
         try_audio_only = true;
-        retry_timer = g_timeout_add (0, cb_retryAudioOnly, this);
+        retry_timer = g_timeout_add (100, cb_retryAudioOnly, this);
         return true;
     }
     return false;
@@ -1079,6 +1072,11 @@ KMPLAYER_NO_EXPORT void OssoMediaServer::processStopped () {
         dbus_message_unref (msg);
         setState (Ready);
     }
+    if (retry_timer) {
+        g_source_remove (retry_timer);
+        retry_timer = 0;
+        try_audio_only = true;
+    }
 }
 
 KMPLAYER_NO_EXPORT void OssoMediaServer::processBuffering (int perc) {
@@ -1129,3 +1127,360 @@ void OssoMediaServer::setSubtitle (int id, const QString &) {
     sendCommand (QString ("quit"));
 }*/
 
+#ifdef __ARMEL__
+
+static NSNotify ns_notify;
+
+extern "C" {
+static void npEmbedded (void *ndata) {
+    ((NpPlayer *) ndata)->pluginEmbedded ();
+}
+
+static void npSetDimension (void *ndata, int w, int h) {
+    NpPlayer *player = (NpPlayer *) ndata;
+    player->setWidth (w);
+    player->setHeight (h);
+    player->updateAspects ();
+}
+
+static void npGetUrl (void *inst, uint32_t stream,
+        const char *url, const char *target, void *ndata) {
+    ((NpPlayer *) ndata)->requestStream (inst, stream, url, target);
+}
+
+static char *nsEvaluate (const char *script, void *ndata) {
+    String result = ((NpPlayer *) ndata)->evaluateScript (script);
+    if (result.isEmpty ())
+        return g_strdup ("");
+    return g_strdup ((const char *)result);
+}
+
+static void npFinish (void *inst, uint32_t stream, void *ndata) {
+    ((NpPlayer *) ndata)->destroyStream (inst, stream);
+}
+
+static gint streamCompare (gconstpointer a, gconstpointer b) {
+    return (long)a - (long)b;
+}
+
+static gboolean deleteStreams (gpointer key, gpointer value, gpointer data) {
+    (void) key;
+    delete (NpStream *)value;
+    return false; // continue
+}
+
+static gboolean copyParams (gpointer key, gpointer value, gpointer data) {
+    char ***tuple = (char ***)data;
+    tuple[0][0] = (char *)key;
+    tuple[1][0] = (char *)value;
+    tuple[0]++;
+    tuple[1]++;
+    return false; // continue
+}
+
+static gboolean openStreams (gpointer key, gpointer value, gpointer data) {
+    int active = (int)(long)data;
+    NpStream *ns = (NpStream *)value;
+    (void) key;
+    if (ns->finish_reason == NpStream::NoReason && !ns->job) {
+        ++active;
+        ns->open ();
+    }
+    return active > 4;
+}
+
+static gboolean findClosedStreams (gpointer key, gpointer value, gpointer data) {
+    NpStream *ns = (NpStream *)value;
+    (void) key;
+    if (ns->finish_reason > NpStream::NoReason) {
+        *(NpStream **)data = ns;
+        return true;
+    }
+    return false;
+}
+
+} // extern "C"
+
+KDE_NO_CDTOR_EXPORT NpPlayer::NpPlayer (ProcessNotify * ctrl)
+ : Process (ctrl, "npp", false),
+   streams (NULL),
+   mime (NULL),
+   npdata (NULL),
+   argc (0),
+   argn (NULL),
+   argv (NULL),
+   processing(false) {
+    plugins = g_tree_new ((GCompareFunc)::strcmp);
+    if (!ns_notify.embedded) {
+        ns_notify.embedded = npEmbedded;
+        ns_notify.setDimension = npSetDimension;
+        ns_notify.getUrl = npGetUrl;
+        ns_notify.evaluate = nsEvaluate;
+        ns_notify.finishStream = npFinish;
+    }
+}
+
+KDE_NO_CDTOR_EXPORT NpPlayer::~NpPlayer () {
+    if (mime)
+        g_free (mime);
+}
+
+KMPLAYER_NO_EXPORT bool NpPlayer::ready (unsigned int viewer) {
+    debugLog () << "NpPlayer::ready" << endl;
+    m_viewer = viewer;
+    if (m_state < Ready) {
+        // only flash for now
+        mime = g_strdup ("application/x-shockwave-flash");
+        const char *lib = "/usr/lib/browser/plugins/libflashplayer.so";
+        NPPluginLib *nplib = (NPPluginLib *) g_tree_search (plugins,
+                (GCompareFunc)::strcmp, mime);
+        if (!nplib) {
+            nplib = nppPluginInit (lib);
+            if (!nplib) {
+                warningLog() << "failed to initialize plugin lib" << endl;
+                return false;
+            }
+            g_tree_insert (plugins, (char *) mime, nplib);
+        }
+        nppInstanceWindowParent (npdata, (void *)(long)viewer);
+        setState (Ready);
+        return true;
+    }
+    return false;
+}
+
+KMPLAYER_NO_EXPORT bool NpPlayer::play (NodePtr node) {
+    debugLog () << "NpPlayer::play" << endl;
+    if (playing ())
+        return true;
+    setState (Buffering);
+    Mrl *mrl = node ? node->mrl () : 0L;
+    if (!mrl) {
+        errorLog() << "Not a Mrl " << (node ? node->nodeName() :"null") << endl;
+        setState (Ready);
+        return false;
+    }
+    String strurl = mrl->absolutePath ();
+    if (strurl.isEmpty ()) {
+        setState (Ready);
+        return false;
+    }
+    streams = g_tree_new (streamCompare);
+    Process::play (node);
+
+    GTree *params = g_tree_new ((GCompareFunc)::strcmp);
+    if (mrl->id == id_node_html_object) {
+        for (Node *n = mrl->firstChild().ptr(); n; n = n->nextSibling().ptr()) {
+            /*if (n->id == KMPlayer::id_node_param) {
+                KMPlayer::Element *e = static_cast <KMPlayer::Element *> (n);
+                String name = e->getAttribute (KMPlayer::StringPool::attr_name);
+                if (!g_tree_lookup (params, (const char *) name))
+                    g_tree_insert (params,
+                            strdup ((const char *) name),
+                            strdup ((const char *) e->getAttribute (KMPlayer::StringPool::attr_value)));
+            } else*/ if (n->id == KMPlayer::id_node_html_embed) {
+                KMPlayer::Element *e = static_cast <KMPlayer::Element *> (n);
+                AttributePtr a = e->attributes ()->first ();
+                for (; a; a = a->nextSibling ()) {
+                    String nm = a->name().toString();
+                    if (!g_tree_lookup (params, (const char *) nm))
+                        g_tree_insert (params,
+                                strdup ((const char *) nm),
+                                strdup ((const char *) a->value ()));
+                }
+            }
+        }
+    } else {
+        g_tree_insert (params, strdup ("SRC"), strdup ((const char *) strurl));
+    }
+    argc = g_tree_nnodes (params);
+    argn = (char **) malloc (argc * sizeof (char *));
+    argv = (char **) malloc (argc * sizeof (char *));
+    char **tuple[2] = { argn, argv };
+    g_tree_traverse (params, copyParams, G_IN_ORDER, tuple);
+    g_tree_destroy (params);
+
+    npdata = nppInstanceOpen (mime, argc, argn, argv, (void *)this, &ns_notify);
+    if (!npdata) {
+        g_tree_destroy (streams);
+        streams = NULL;
+        setState (Ready);
+        warningLog() << "failed to create new instance" << endl;
+        return false;
+    }
+    nppInstanceStart ((const char *)strurl, mime, npdata);
+    return true;
+}
+
+KMPLAYER_NO_EXPORT void NpPlayer::pluginEmbedded () {
+    setState (Playing);
+}
+
+KMPLAYER_NO_EXPORT void NpPlayer::requestStream (void *inst, uint32_t sid,
+        const String & url, const String & target) {
+    URL uri (m_url, url);
+    debugLog () << "NpPlayer::request " << sid << " '" << uri << "'" << endl;
+    if (!target.isEmpty ()) {
+        debugLog () << "new page request " << target << endl;
+        if (url.hasPrefix ("javascript:")) {
+            String result = evaluateScript (url.mid (11));
+            debugLog() << "result is " << result << endl;
+            if (result == "undefined")
+                uri = URL ();
+            else
+                uri = URL (m_url, result); // probably wrong ..
+        }
+        /*if (!uri.url ().isEmpty ())
+            emit openUrl (uri, target);*/
+        nppStreamFinished (sid, NpStream::BecauseDone, this);
+    } else if (url.hasPrefix ("javascript:")) {
+        String result = evaluateScript (url.mid (11));
+        const char *data = (const char *)result;
+        nppStreamData (sid, data, strlen (data), this);
+        nppStreamFinished (sid, NpStream::BecauseDone, this);
+    } else {
+        NpStream * ns = new NpStream (this, sid, uri);
+        g_tree_insert (streams, (gpointer) sid, ns);
+        processStreams ();
+    }
+}
+
+KMPLAYER_NO_EXPORT void NpPlayer::destroyStream (void *inst, uint32_t sid) {
+    NpStream * ns = (NpStream *) g_tree_lookup (streams, (gpointer) sid);
+    if (ns) {
+        nppStreamFinished (sid, NpStream::BecauseStopped, this);
+        g_tree_remove (streams, (gpointer) sid);
+        delete (ns);
+    }
+}
+
+KMPLAYER_NO_EXPORT bool NpPlayer::stop () {
+    if (npdata) {
+        g_tree_foreach (streams, deleteStreams, NULL);
+        g_tree_destroy (streams);
+        streams = NULL;
+        nppInstanceClose (npdata);
+        npdata = NULL;
+        setState (Ready);
+    }
+    for (int i = 0; i < argc; i++) {
+        free (argn[i]);
+        free (argv[i]);
+    }
+    if (argn) {
+        free (argn);
+        free (argv);
+        argn = argv = NULL;
+    }
+    argc = 0;
+    return true;
+}
+
+KMPLAYER_NO_EXPORT bool NpPlayer::quit () {
+    stop ();
+    if (mime)
+        g_free (mime);
+    mime = NULL;
+    nppInstanceWindowClose (npdata); //FIXME
+    setState (NotRunning);
+    return true;
+}
+
+KMPLAYER_NO_EXPORT
+void NpPlayer::setVideoWindow (int x, int y, int w, int h) {
+    debugLog() << "NpPlayer::setVideoWindow " << w << "x" << h << endl;
+    if (!(x < 0 || y < 0 || w <= 0 || h <= 1 /* hack !!*/ ||
+                x > 800 || y > 480 || w > 800 || h > 480))
+        nppInstanceWindowDimension (npdata, x, y, w, h);
+}
+
+KMPLAYER_NO_EXPORT void NpPlayer::processStreams () {
+    if (!processing) {
+        processing = true;
+        streams_deleted = 0;
+        int active = 0;
+        g_tree_traverse (streams, openStreams, G_IN_ORDER, &active);
+        while (streams_deleted > 0) {
+            NpStream *ns = NULL;
+            g_tree_traverse (streams, findClosedStreams, G_IN_ORDER, &ns);
+            if (ns) {
+                g_tree_remove (streams, (gpointer) ns->stream);
+                delete ns;
+            } else {
+                errorLog() << "findClosedStreams didn't find a stream" << endl;
+                break;
+            }
+            streams_deleted--;
+        }
+        processing = false;
+    }
+}
+
+KMPLAYER_NO_EXPORT void NpPlayer::streamFinished (NpStream *ns) {
+    nppStreamFinished (ns->stream, ns->finish_reason, this);
+    if (!processing) {
+        g_tree_remove (streams, (gpointer) ns->stream);
+        delete ns;
+        processStreams ();
+    } else {
+        streams_deleted++;
+    }
+}
+
+KMPLAYER_NO_EXPORT String NpPlayer::evaluateScript (const String &script) {
+    debugLog() << "evaluateScript " << script << endl;
+    if (script == "window.location+\"__flashplugin_unique__\"" ||
+            script == "top.location+\"__flashplugin_unique__\"")
+        return "http://www.example.com/__flashplugin_unique__";
+    return "undefined";
+}
+
+KDE_NO_CDTOR_EXPORT NpStream::NpStream (NpPlayer *p, uint32_t id, const URL & u)
+ : player (p), stream (id), url (u), job (NULL), finish_reason (NoReason) {
+}
+
+KDE_NO_CDTOR_EXPORT NpStream::~NpStream () {
+    close ();
+}
+
+KMPLAYER_NO_EXPORT void NpStream::open () {
+    debugLog () << "NpStream::open " << url.url () << endl;
+    if (url.url().hasPrefix ("javascript:")) {
+        String result = player->evaluateScript (url.url().mid (11));
+        const char *data = (const char *)result;
+        nppStreamData (stream, data, strlen (data), NULL /*FIXME*/);
+        finish_reason = BecauseDone;
+        player->streamFinished (this);
+    } else {
+        bytes = 0;
+        job = asyncGet (this, url.url ());
+    }
+}
+
+KMPLAYER_NO_EXPORT void NpStream::close () {
+    if (job)
+        job->kill ();
+    job = NULL;
+}
+
+KMPLAYER_NO_EXPORT void NpStream::jobData (Job *jb, ByteArray & data) {
+    //debugLog () << "NpStream::jobData " << url.url () << endl;
+    if (!bytes)
+        nppStreamInfo (stream, jb->contentType (), jb->contentLength ());
+    bytes += data.size ();
+    nppStreamData (stream, data.data (), data.size (), NULL /*FIXME*/);
+}
+
+KMPLAYER_NO_EXPORT void NpStream::jobResult (Job *jb) {
+    debugLog () << "NpStream::jobResult " << bytes << endl;
+    finish_reason = jb->error () || !bytes ? BecauseError : BecauseDone;
+    job = NULL;
+    player->streamFinished (this);
+}
+
+KMPLAYER_NO_EXPORT void NpStream::redirected (Job *, const String & uri) {
+    url = URL (url, uri);
+    nppStreamRedirected (stream, (const char *)url.url ());
+}
+
+#endif //__ARMEL__
