@@ -30,6 +30,8 @@
 #include <libgnomevfs/gnome-vfs-uri.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
 #include <libgnomevfs/gnome-vfs-mime-utils.h>
+#include <libgnomevfs/gnome-vfs-module-callback.h>
+#include <libgnomevfs/gnome-vfs-standard-callbacks.h>
 
 #include "kmplayertypes.h"
 
@@ -41,8 +43,10 @@ KMPLAYER_NO_EXPORT void ByteArrayImpl::resize (unsigned ns) {
         if (m_size > 0)
             memcpy (tmp, m_data, m_size);
         m_capacity = m_size = ns;
-        free (m_data);
+        if (m_data && data_owner)
+            free (m_data);
         m_data = tmp;
+        data_owner = true;
     } else
         m_size = ns;
 }
@@ -512,7 +516,7 @@ String GLibTextStream::readLine () {
 }
 
 //-----------------------------------------------------------------------------
-
+#if 0
 namespace KMPlayer {
 
     class GnomeVFSGetJob : public JobInterface {
@@ -521,10 +525,12 @@ namespace KMPlayer {
         ~GnomeVFSGetJob ();
         void kill (bool quiet = true);
         void finished ();
+        bool error ();
 
         GnomeVFSAsyncHandle * network_handle;
         AsyncJobInterface * m_receiver;
         bool job_closed;
+        bool job_error;
         bool keep_quiet;
         bool continuing;
         char read_buf [2048];
@@ -535,6 +541,28 @@ namespace KMPlayer {
     }
 }
 
+//static GTree *uri_info;
+
+//void getRedirection (gpointer data, gpointer d) {
+//    debugLog() << " getRedirection: " << (char *)data << endl;
+//}
+
+//static void globalVFSAsyncModuleCallback (gconstpointer in,
+//        gsize in_size,
+//        gpointer out,
+//        gsize out_size,
+//        gpointer callback_data/*,
+//        GnomeVFSModuleCallbackResponse response,
+//        gpointer response_data*/) {
+//    GnomeVFSModuleCallbackReceivedHeadersIn *in_args =
+//        (GnomeVFSModuleCallbackReceivedHeadersIn *)in;
+//    debugLog() << "globalVFSAsyncModuleCallback " << 
+//        in_args->uri->text << endl;
+//    g_list_foreach (in_args->headers, getRedirection, NULL);
+//    //response (response_data);
+//}
+
+
 static void cb_async_get_close (GnomeVFSAsyncHandle *handle, GnomeVFSResult result, gpointer p) {
     //debugLog () << "cb_async_get_close " << endl;
     if (!p)
@@ -549,6 +577,7 @@ static void cb_async_get_read (GnomeVFSAsyncHandle *handle, GnomeVFSResult resul
         return;
     get->continuing = true;
     if (bytes_read > 0) {
+    debugLog () << "cb_async_get_read " << handle << endl;
         ByteArray data (bytes_read);
         memcpy (data.data (), (const char *) buffer, bytes_read);
         get->m_receiver->jobData (get, data);
@@ -557,6 +586,7 @@ static void cb_async_get_read (GnomeVFSAsyncHandle *handle, GnomeVFSResult resul
     if (result != GNOME_VFS_OK || bytes_read <= 0 || !get->continuing) {
         // || bytes_read < bytes_requested 
         get->job_closed = true;
+        get->job_error = result != GNOME_VFS_ERROR_EOF;
         gnome_vfs_async_cancel (handle);
         gnome_vfs_async_close (handle, cb_async_get_close, p);
     } else {
@@ -566,7 +596,7 @@ static void cb_async_get_read (GnomeVFSAsyncHandle *handle, GnomeVFSResult resul
 }
 
 static void cb_async_get_open (GnomeVFSAsyncHandle *handle, GnomeVFSResult result, gpointer p) {
-    //debugLog () << "cb_async_get_open " << endl;
+    debugLog () << "cb_async_get_open " << handle << endl;
     GnomeVFSGetJob * get = static_cast <GnomeVFSGetJob *> (p);
     if (get->job_closed)
         return;
@@ -578,8 +608,19 @@ static void cb_async_get_open (GnomeVFSAsyncHandle *handle, GnomeVFSResult resul
 
 GnomeVFSGetJob::GnomeVFSGetJob (AsyncJobInterface * rec, const String &url)
  : m_receiver (rec), network_handle (0L),
-   job_closed (false), keep_quiet (false), continuing (false) {
+   job_closed (false), job_error (false),
+   keep_quiet (false), continuing (false) {
+
+    //if (!uri_info) {
+    //    uri_info = g_tree_new ((GCompareFunc)::strcmp);
+    //    gnome_vfs_module_callback_set_default
+    //        (GNOME_VFS_MODULE_CALLBACK_HTTP_RECEIVED_HEADERS,
+    //         globalVFSAsyncModuleCallback,
+    //         uri_info,
+    //         NULL);
+    //}
     gnome_vfs_async_open (&network_handle, (const char *) url, GNOME_VFS_OPEN_READ, 0, cb_async_get_open , static_cast <void *> (this));
+    debugLog() << "GnomeVFSGetJob " << network_handle << " " << url << endl;
 }
 
 GnomeVFSGetJob::~GnomeVFSGetJob () {
@@ -609,6 +650,269 @@ void GnomeVFSGetJob::finished () {
     delete this;
 }
 
+bool GnomeVFSGetJob::error () {
+    return job_error;
+}
+#endif
+//-----------------------------------------------------------------------------
+
+#include <curl/curl.h>
+
+static gint easyCompare (gconstpointer a, gconstpointer b) {
+    return (long)a - (long)b;
+}
+
+namespace KMPlayer {
+
+    struct FdWatch {
+        int fd;
+        int chan_read_watch;
+    };
+    class KMPLAYER_NO_EXPORT CurlGetJob : public JobInterface {
+    public:
+        CurlGetJob (AsyncJobInterface * rec, const String & u);
+        ~CurlGetJob ();
+        void kill (bool quiet = true);
+        bool error ();
+        unsigned long contentLength() const { return content_length; }
+        String contentType () const { return content_type; }
+
+        AsyncJobInterface * m_receiver;
+
+        static bool global_init;
+        static CURLM *multi;
+
+        String url;
+        String content_type;
+        CURL *easy;
+        bool quiet;
+        bool killed;
+        bool redirected;
+        int error_code;
+        unsigned long content_length;
+    };
+
+    Job * asyncGet (AsyncJobInterface * receiver, const String & url) {
+        return new CurlGetJob (receiver, url);
+    }
+}
+
+static gboolean findKilledJob (gpointer key, gpointer value, gpointer data) {
+    CurlGetJob *job = (CurlGetJob *)value;
+    if (job->killed) {
+        *(CurlGetJob **)data = job;
+        return true;
+    }
+    return false;
+}
+
+static gboolean curlChannelRead (GIOChannel *src, GIOCondition c, gpointer d);
+bool CurlGetJob::global_init = false;
+CURLM *CurlGetJob::multi = NULL;
+static GTree *easy_list = NULL;
+static FdWatch *fd_watch_array;
+static int fd_watch_array_len;
+static gint curl_timeout_timer;
+static bool in_perform_multi;
+static bool curl_job_killed;
+
+static void curlChannelDestroy (gpointer) {
+    debugLog() << "channel destoyed" << endl;
+}
+
+static void updateReadWatches() {
+    fd_set fd_read, fd_write, fd_excep;
+    FD_ZERO (&fd_read);
+    FD_ZERO (&fd_write);
+    FD_ZERO (&fd_excep);
+    int maxfd = 0;
+    curl_multi_fdset (CurlGetJob::multi, &fd_read, &fd_write, &fd_excep, &maxfd);
+    if (maxfd > 0 && !fd_watch_array) {
+        fd_watch_array = new FdWatch[maxfd];
+        memset (fd_watch_array, 0, maxfd * sizeof (FdWatch));
+        fd_watch_array_len = maxfd;
+    } else if (maxfd > fd_watch_array_len) {
+        FdWatch *tmp = fd_watch_array;
+        fd_watch_array = new FdWatch[maxfd];
+        memset (fd_watch_array, 0, maxfd * sizeof (FdWatch));
+        memcpy (fd_watch_array, tmp, fd_watch_array_len * sizeof (FdWatch));
+        delete[] tmp;
+        fd_watch_array_len = maxfd;
+    }
+    for (int i = 3; i < fd_watch_array_len; i++) {
+        if (FD_ISSET (i, &fd_read)) {
+            if (!fd_watch_array[i].fd) {
+                fd_watch_array[i].fd = i;
+                GIOChannel *channel = g_io_channel_unix_new (i);
+                g_io_channel_set_encoding (channel, NULL, NULL);
+                fd_watch_array[i].chan_read_watch = g_io_add_watch_full (channel, G_PRIORITY_DEFAULT, G_IO_IN, curlChannelRead, NULL, curlChannelDestroy);
+                g_io_channel_unref (channel);
+            }
+        } else if (fd_watch_array[i].fd) {
+            g_source_remove (fd_watch_array[i].chan_read_watch);
+            fd_watch_array[i].fd = fd_watch_array[i].chan_read_watch = 0;
+        }
+    }
+}
+
+static void curlPerformMulti () {
+
+    in_perform_multi = true;
+
+    int running_handles = 0;
+    CURLMcode curlCode;
+    do {
+        curl_job_killed = false;
+        curlCode = curl_multi_perform (CurlGetJob::multi, &running_handles);
+	if (curl_job_killed)
+            do {
+                CurlGetJob *job = NULL;
+                g_tree_traverse (easy_list, findKilledJob, G_IN_ORDER, &job);
+                if (job)
+                    delete job;
+                else
+                    break;
+            } while (true);
+    } while (CURLM_CALL_MULTI_PERFORM == curlCode && running_handles > 0);
+    int messages;
+    CURLMsg* msg = curl_multi_info_read (CurlGetJob::multi, &messages);
+    while (msg) {
+        if (CURLMSG_DONE == msg->msg) {
+            CurlGetJob *job = (CurlGetJob *)g_tree_lookup (easy_list, msg->easy_handle);
+            if (job)
+                delete job;
+        }
+        msg = curl_multi_info_read (CurlGetJob::multi, &messages);
+    }
+    updateReadWatches();
+
+    in_perform_multi = false;
+}
+
+static gboolean curlTimeOut (void *) {
+    curlPerformMulti ();
+    if (g_tree_height (easy_list))
+        return 1; // continue
+    curl_timeout_timer = 0;
+    debugLog() << "clearing polling" << endl;
+    return 0; // stop
+}
+
+static gboolean curlChannelRead (GIOChannel *src, GIOCondition c, gpointer d) {
+    curlPerformMulti ();
+}
+
+static 
+size_t curlHeaderLine( void *ptr, size_t size, size_t nmemb, void *stream) {
+    CurlGetJob *job = (CurlGetJob *) stream;
+    //char buf[256];
+    //snprintf (buf, sizeof (buf), "%s", (char *)ptr);
+    //debugLog() << "header: " << buf << endl;
+    // Last-Modified: 
+    if (!job->killed) {
+        if (!strncmp ((char *)ptr, "HTTP/", 5)) {
+            char *p = strchr ((char *)ptr + 5, ' ');
+            if (p) {
+                int rc = strtol (p +1, NULL, 10);
+                if (rc >= 300)
+                    job->redirected = true;
+                else if (rc >= 400)
+                    job->error_code = rc;
+            }
+        } else if (job->redirected && !strncmp ((char *)ptr, "Location:", 9)) {
+            String nuri ((char *)ptr+9);
+            job->m_receiver->redirected (job, nuri.stripWhiteSpace ());
+            job->redirected = false;
+        } else if (!strncmp ((char *)ptr, "Content-Length:", 15)) {
+            job->content_length = strtol ((char *)ptr + 15, NULL, 10);
+        } else if (!strncmp ((char *)ptr, "Content-Type:", 13)) {
+            job->content_type = String ((char *)ptr + 13).stripWhiteSpace ();
+        }
+    }
+    return size * nmemb;
+}
+
+static 
+size_t curlDataReceived (void *ptr, size_t size, size_t nmemb, void *stream) {
+    CurlGetJob *job = (CurlGetJob *) stream;
+    ByteArray data ((char *)ptr, size * nmemb);
+    if (!job->killed)
+        job->m_receiver->jobData (job, data);
+    return size * nmemb;
+}
+static int job_count;
+
+KMPLAYER_NO_CDTOR_EXPORT
+CurlGetJob::CurlGetJob (AsyncJobInterface * rec, const String & u)
+ : m_receiver (rec), url (u),
+   quiet (false), killed (false), redirected (false),
+   error_code (0), content_length (0) {
+    if (!global_init) {
+        global_init = !curl_global_init (CURL_GLOBAL_ALL);
+        easy_list = g_tree_new (easyCompare);
+        debugLog() << "global curl init: " << global_init << endl;
+    }
+    if (!multi)
+        multi = curl_multi_init ();
+    easy = curl_easy_init ();
+    debugLog() << "CurlGetJob::CurlGetJob " << easy << " " << ++job_count << endl;
+    g_tree_insert (easy_list, easy, this);
+    //curl_easy_setopt (easy, CURLOPT_VERBOSE, true);
+    curl_easy_setopt (easy, CURLOPT_NOPROGRESS, true);
+    curl_easy_setopt (easy, CURLOPT_WRITEFUNCTION, curlDataReceived);
+    curl_easy_setopt (easy, CURLOPT_WRITEDATA, this);
+    curl_easy_setopt (easy, CURLOPT_HEADERFUNCTION, curlHeaderLine);
+    curl_easy_setopt (easy, CURLOPT_WRITEHEADER, this);
+    curl_easy_setopt (easy, CURLOPT_FOLLOWLOCATION, 1);
+    curl_easy_setopt (easy, CURLOPT_MAXREDIRS, 10);
+    //CURLSH *curl_share_init( );
+    //curl_easy_setopt (easy, CURLOPT_SHARE, m_curlShareHandle);
+    //curl_share_setopt(share_handle,CURL_LOCK_DATA_DNS);
+    //curl_share_cleanup(CURLSH * share_handle );
+    curl_easy_setopt (easy, CURLOPT_DNS_CACHE_TIMEOUT, 60 * 5);
+    curl_easy_setopt (easy, CURLOPT_ENCODING, "");
+    curl_easy_setopt (easy, CURLOPT_USERAGENT, "Mozilla/4.0 (compatible; MSIE 6.0; X11; Linux armv6l; U) Opera 8.5 [en_US] Tablet browser 0.0.14 RX-34_2007SE_4.2007.26-8");
+    curl_easy_setopt (easy, CURLOPT_SSL_VERIFYPEER, 0);
+    curl_easy_setopt (easy, CURLOPT_URL, (const char *)url);
+    //if (!strncmp (u, "http", 4))
+    //    curl_easy_setopt (easy, CURLOPT_HTTPGET, true);
+    CURLMcode type = curl_multi_add_handle (multi, easy);
+    if (type && type != CURLM_CALL_MULTI_PERFORM)
+        debugLog() << "CurlGetJob add to multi failure" << endl;
+
+    if (!curl_timeout_timer)
+        curl_timeout_timer = g_timeout_add (50, curlTimeOut, this);
+}
+
+KMPLAYER_NO_CDTOR_EXPORT CurlGetJob::~CurlGetJob () {
+    debugLog() << "CurlGetJob::~CurlGetJob " << easy << " " << --job_count << endl;
+    if (easy) {
+        curl_multi_remove_handle (multi, easy);
+        curl_easy_cleanup (easy);
+        g_tree_remove (easy_list, easy);
+        if (!quiet)
+            m_receiver->jobResult (this);
+    }
+    if (0) {
+        curl_multi_cleanup (multi);
+        multi = NULL;
+    }
+}
+
+KMPLAYER_NO_EXPORT void CurlGetJob::kill (bool q) {
+    quiet = q;
+    killed = true;
+    curl_job_killed = true;
+    if (!in_perform_multi) {
+        delete this;
+	curlPerformMulti ();
+    }
+}
+
+KMPLAYER_NO_EXPORT bool CurlGetJob::error () {
+    return false;
+}
+
 //-----------------------------------------------------------------------------
 
 static bool gconf_initialized = false;
