@@ -28,6 +28,7 @@
 #include <glib.h>
 #include <gdk/gdk.h>
 #include <gconf/gconf-client.h>
+#include <libgnomevfs/gnome-vfs-mime-utils.h>
 
 #undef KDE_NO_CDTOR_EXPORT
 #undef KDE_NO_EXPORT
@@ -185,6 +186,7 @@ public:
     ByteArrayTmpl ();
     ByteArrayTmpl (unsigned sz);
     ByteArrayTmpl (const ByteArrayTmpl <T> & other);
+    ByteArrayTmpl (const char * data, unsigned size);
     ~ByteArrayTmpl ();
     unsigned size () const;
     void resize (unsigned ns);
@@ -198,10 +200,12 @@ class ByteArrayImpl {
     char * m_data;
     unsigned m_size;
     unsigned m_capacity;
+    bool data_owner;
 public:
     ByteArrayImpl ();
     ByteArrayImpl (unsigned sz);
     ByteArrayImpl (const ByteArrayImpl & other);
+    ByteArrayImpl (const char * data, unsigned size);
     ~ByteArrayImpl ();
     unsigned size () const;
     void resize (unsigned ns);
@@ -468,19 +472,18 @@ template <class T>
 class MimeTypeTmpl {
 public:
     static String findByContent (const char * data, int size);
-    //static String findByURL (const URL & url);
+    static String findByURL (const URL & url);
 };
 
 class GMimeType {
 public:
     static String findByContent (const char * data, int size);
-    //static String findByURL (const URL & url);
+    static String findByURL (const URL & url);
 };
 
 typedef MimeTypeTmpl <GMimeType> MimeType;
 
 
-class GnomeVFSJobPrivate;
 class JobInterface;
 class AsyncJobInterface;
 typedef JobInterface Job;
@@ -489,8 +492,12 @@ Job * asyncGet (AsyncJobInterface * receiver, const String & url);
 
 class JobInterface {
 public:
-    virtual void kill (bool quiet = true) = 0;
     virtual ~JobInterface () {};
+
+    virtual void kill (bool quiet = true) = 0;
+    virtual bool error () = 0;
+    virtual unsigned long contentLength() const { return 0; }
+    virtual String contentType () const { return String(); }
 protected:
     JobInterface () {}
 };
@@ -500,6 +507,7 @@ public:
     virtual ~AsyncJobInterface () {}
     virtual void jobData (Job * job, ByteArray & data) = 0;
     virtual void jobResult (Job * job) = 0;
+    virtual void redirected (Job * job, const String & uri) {};
 };
 
 template <class T>
@@ -1038,6 +1046,10 @@ template <class T>
 inline ByteArrayTmpl <T>::ByteArrayTmpl (const ByteArrayTmpl <T> & b)
     : m_bytearray (b.m_bytearray) {}
 
+template <class T>
+inline ByteArrayTmpl <T>::ByteArrayTmpl (const char * d, unsigned sz)
+    : m_bytearray (d, sz) {}
+
 template <class T> inline ByteArrayTmpl <T>::~ByteArrayTmpl () {}
 
 template <class T> inline unsigned ByteArrayTmpl <T>::size () const {
@@ -1511,10 +1523,10 @@ inline String MimeTypeTmpl <T>::findByContent (const char * data, int size) {
     return T::findByContent (data, size);
 }
 
-/*template <class T>
+template <class T>
 inline String MimeTypeTmpl <T>::findByURL (const URL & url) {
     return T::findByURL (url);
-}*/
+}
 
 //-----------------------------------------------------------------------------
 
@@ -1840,21 +1852,28 @@ inline void PainterTmpl <T>::drawImage (int dx, int dy, const Image & im, int sx
 //-----------------------------------------------------------------------------
 
 KMPLAYER_NO_CDTOR_EXPORT inline ByteArrayImpl::ByteArrayImpl ()
- : m_data (0L), m_size (0), m_capacity (0) {}
+ : m_data (0L), m_size (0), m_capacity (0), data_owner (true) {}
 
 KMPLAYER_NO_CDTOR_EXPORT inline ByteArrayImpl::ByteArrayImpl (unsigned sz)
- : m_data ((char *) malloc (sz)), m_size (sz), m_capacity (sz) {}
+ : m_data ((char *) malloc (sz)),
+   m_size (sz),
+   m_capacity (sz),
+   data_owner (true) {}
 
 KMPLAYER_NO_CDTOR_EXPORT inline ByteArrayImpl::ByteArrayImpl (const ByteArrayImpl & b)
  : m_data (b.m_size ? (char *) malloc (b.m_size) : 0L),
    m_size (b.m_size),
-   m_capacity (b.m_size) {
+   m_capacity (b.m_size),
+   data_owner (true) {
     if (m_size)
         memcpy (m_data, b.m_data, m_size);
 }
 
+inline ByteArrayImpl::ByteArrayImpl (const char * d, unsigned sz)
+ : m_data ((char *)d), m_size (sz), m_capacity (sz), data_owner (false) {}
+
 KMPLAYER_NO_CDTOR_EXPORT inline ByteArrayImpl::~ByteArrayImpl () {
-    if (m_data)
+    if (m_data && data_owner)
         free (m_data);
 }
 
@@ -2035,9 +2054,9 @@ KMPLAYER_NO_CDTOR_EXPORT inline GnomeVFSUrl::GnomeVFSUrl () : m_vfsurl (0L) {}
 
 //-----------------------------------------------------------------------------
 
-/*inline String GMimeType::findByURL (const URL & url) {
+inline String GMimeType::findByURL (const URL & url) {
     return String (gnome_vfs_get_mime_type ((const char *) url.url ()));
-}*/
+}
 
 //-----------------------------------------------------------------------------
 
