@@ -211,9 +211,7 @@ KDE_NO_EXPORT void ViewSurface::video (Single x, Single y, Single w, Single h) {
 
 //-------------------------------------------------------------------------
 
-//# define USE_CAIRO_GLITZ
-
-static cairo_surface_t * cairoCreateX11Surface (GtkWidget * wid, int w, int h) {
+static cairo_surface_t * cairoCreateSurface (GtkWidget * wid, int w, int h) {
     GdkDisplay * gdisplay = gtk_widget_get_display (wid);
     Display * display = gdk_x11_display_get_xdisplay (gdisplay);
     Window id = gdk_x11_drawable_get_xid (GDK_DRAWABLE (wid->window));
@@ -228,86 +226,6 @@ static cairo_surface_t * cairoCreateX11Surface (GtkWidget * wid, int w, int h) {
                     DefaultScreen (qt_xdisplay ()))),
             w, h);*/
 }
-# ifndef USE_CAIRO_GLITZ
-cairo_surface_t * (*cairoCreateSurface)(GtkWidget*, int, int)=cairoCreateX11Surface;
-# else
-
-#  include <cairo-glitz.h>
-#  include <glitz.h>
-#  include <glitz-glx.h>
-
-glitz_drawable_t * glitz_drawable = 0L; // FIXME add in ViewArea
-
-static cairo_surface_t * cairoCreateSurface (Window id, int w, int h) {
-    glitz_drawable_format_t  formatDrawableTemplate;
-    formatDrawableTemplate.doublebuffer = 1;
-    unsigned long mask = GLITZ_FORMAT_DOUBLEBUFFER_MASK;
-    glitz_drawable_format_t* draw_fmt = NULL;
-    glitz_drawable_format_t* fmt = 0L;
-    XVisualInfo* pVisualInfo;
-    VisualID vis_id = XVisualIDFromVisual (DefaultVisual (qt_xdisplay (),
-                DefaultScreen (qt_xdisplay ())));
-    int i = 0;
-    do {
-        fmt = glitz_glx_find_window_format (
-                qt_xdisplay (),
-                DefaultScreen (qt_xdisplay ()),
-                mask,
-                &formatDrawableTemplate,
-                i++);
-        if (fmt) {
-            pVisualInfo = glitz_glx_get_visual_info_from_format (
-                    qt_xdisplay (), DefaultScreen (qt_xdisplay()), fmt);
-            kdDebug() << "depth = " << pVisualInfo->depth << " id:" << pVisualInfo->visualid << endl;
-            if (pVisualInfo->visualid == vis_id) {
-            //if (pVisualInfo->depth == 32) {
-                  //DefaultDepth(qt_xdisplay(), DefaultScreen(qt_xdisplay()))) {
-                draw_fmt = fmt;
-                break;
-            } else if (!draw_fmt)
-                draw_fmt = fmt;
-        }
-    } while (fmt);
-    if (!draw_fmt)
-        return cairoCreateX11Surface (id, w, h);
-
-    glitz_drawable_t * drawable = glitz_glx_create_drawable_for_window (
-            qt_xdisplay (), DefaultScreen (qt_xdisplay()), draw_fmt, id, w, h);
-    if (!drawable) {
-        kdWarning() << "failed to create glitz drawable on screen " << DefaultScreen (qt_xdisplay()) << endl;
-        return cairoCreateX11Surface (id, w, h);
-    }
-
-    glitz_format_t formatTemplate;
-    formatTemplate.color = draw_fmt->color;
-    formatTemplate.color.fourcc = GLITZ_FOURCC_RGB;
-
-    glitz_format_t* format = glitz_find_format (drawable,
-            GLITZ_FORMAT_RED_SIZE_MASK   |
-            GLITZ_FORMAT_GREEN_SIZE_MASK |
-            GLITZ_FORMAT_BLUE_SIZE_MASK  |
-            GLITZ_FORMAT_ALPHA_SIZE_MASK |
-            GLITZ_FORMAT_FOURCC_MASK,
-            &formatTemplate,
-            0);
-    //glitz_format_t* format = glitz_find_standard_format (
-    //        drawable, GLITZ_STANDARD_ARGB32);
-    glitz_surface_t * pGlitzSurface = glitz_surface_create (
-            drawable, format, w, h, 0, NULL);
-    if (draw_fmt->doublebuffer)
-        glitz_surface_attach (pGlitzSurface,
-                drawable,
-                GLITZ_DRAWABLE_BUFFER_BACK_COLOR);
-    else
-        glitz_surface_attach (pGlitzSurface,
-                drawable,
-                GLITZ_DRAWABLE_BUFFER_FRONT_COLOR);
-    cairo_surface_t* pCairoSurface = cairo_glitz_surface_create (pGlitzSurface);
-    glitz_surface_destroy (pGlitzSurface);
-    glitz_drawable = drawable;
-    return pCairoSurface;
-}
-# endif
 
 #define CAIRO_SET_SOURCE_RGB(cr,c)           \
     cairo_set_source_rgb ((cr),               \
@@ -345,7 +263,9 @@ public:
 
 KDE_NO_CDTOR_EXPORT
 CairoPaintVisitor::CairoPaintVisitor (cairo_surface_t * cs, const SRect & rect)
- : clip (rect), cairo_surface_widget (cs) {
+ : clip (rect),
+#if 0
+ cairo_surface_widget (cs) {
     cairo_surface = cairo_image_surface_create
         (CAIRO_FORMAT_RGB24, rect.width(), rect.height());
     cr = cairo_create (cairo_surface);
@@ -355,10 +275,22 @@ CairoPaintVisitor::CairoPaintVisitor (cairo_surface_t * cs, const SRect & rect)
     cairo_translate (cr, -rect.x(), -rect.y());
     //cairo_set_operator (cr, CAIRO_OPERATOR_OVER);
     cairo_set_tolerance (cr, 0.5 );
+#else
+ cairo_surface (cs) {
+    cr = cairo_create (cs);
+    cairo_rectangle (cr, rect.x(), rect.y(), rect.width(), rect.height());
+    cairo_clip (cr);
+    cairo_set_operator (cr, CAIRO_OPERATOR_OVER);
+    cairo_set_tolerance (cr, 0.5 );
+    cairo_push_group (cr);
+    cairo_set_source_rgb (cr, 0, 0, 0);
+    cairo_rectangle (cr, rect.x(), rect.y(), rect.width(), rect.height());
+    cairo_fill (cr);
+#endif
 }
 
 KDE_NO_CDTOR_EXPORT CairoPaintVisitor::~CairoPaintVisitor () {
-#ifndef USE_CAIRO_GLITZ
+#if 0
     cairo_pattern_t * pat = cairo_pattern_create_for_surface (cairo_surface);
     cairo_surface_destroy (cairo_surface);
     cairo_pattern_set_extend (pat, CAIRO_EXTEND_NONE);
@@ -372,9 +304,11 @@ KDE_NO_CDTOR_EXPORT CairoPaintVisitor::~CairoPaintVisitor () {
     cairo_destroy (wcr);
     cairo_pattern_destroy (pat);
 #else
-    cairo_surface_flush (cairo_surface);
-    if (glitz_drawable)
-        glitz_drawable_swap_buffers (glitz_drawable);
+    cairo_pattern_t * pat = cairo_pop_group (cr);
+    cairo_set_source (cr, pat);
+    cairo_rectangle (cr, clip.x(), clip.y(), clip.width(), clip.height());
+    cairo_fill (cr);
+    cairo_pattern_destroy (pat);
 #endif
     cairo_destroy (cr);
 }
@@ -484,63 +418,39 @@ KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::Region * reg) {
 KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::ImageMediaType * img) {
     //debugLog() << "Visit " << img->nodeName()  << endl;
     ImageRuntime * ir = static_cast <ImageRuntime *> (img->runtime ());
-    SMIL::RegionBase * rb = convertNode <SMIL::RegionBase> (img->region_node);
     ImageData * id = ir->cached_img.data.ptr ();
-    if (rb && rb->surface && id && !id->isEmpty () && img->keepContent (img)) {
-        SRect rect = rb->surface->bounds;
-        Single x, y, w = rect.width(), h = rect.height();
-        if (id && !id->isEmpty () && id->width() > 0 && id->height() > 0 &&
-                (int)w > 0 && (int)h > 0) {
-            float xs = 1.0, ys = 1.0;
-            if (ir->fit == fit_meet) {
-                float pasp = 1.0 * id->width() / id->height();
-                float rasp = 1.0 * w / h;
-                if (pasp > rasp)
-                    xs = ys = 1.0 * w / id->width();
-                else
-                    xs = ys = 1.0 * h / id->height();
-            } else if (ir->fit == fit_fill) {
-                xs = 1.0 * w / id->width();
-                ys = 1.0 * h / id->height();
-            } else if (ir->fit == fit_slice) {
-                float pasp = 1.0 * id->width() / id->height();
-                float rasp = 1.0 * w / h;
-                if (pasp > rasp)
-                    xs = ys = 1.0 * h / id->height();
-                else
-                    xs = ys = 1.0 * w / id->width();
-            } // else fit_hidden
-            matrix.getXYWH (x, y, w, h);
-            SRect reg_rect (x, y, w, h);
-            x = y = 0;
-            Single w1 = w = xs * id->width();
-            Single h1 = h = ys * id->height();
-            ir->sizes.calcSizes(img, rect.width(), rect.height(), x, y, w1, h1);
-            img->cached_rect = SRect (x, y, w, h);
-            matrix.getXYWH (x, y, w, h);
-            SRect clip_rect = clip.intersect (
-                    reg_rect.intersect (SRect (x, y, w, h)));
-            if (!clip_rect.isValid ())
-                return;
-            cairo_pattern_t * pat = id->cairoImage (w, h, cairo_surface);
-            if (pat) {
-                cairo_matrix_t mat;
-                cairo_matrix_init_identity (&mat);
-                cairo_matrix_translate (&mat, -x, -y);
-                cairo_pattern_set_matrix (pat, &mat);
-                cairo_pattern_set_filter (pat, CAIRO_FILTER_FAST);
-                cairo_set_source (cr, pat);
-                cairo_rectangle (cr, clip_rect.x (), clip_rect.y (),
-                        clip_rect.width (), clip_rect.height ());
-                if (img->trans_steps > img->trans_step) { // only fadein for now
-                    cairo_save (cr);
-                    cairo_clip (cr);
-                    cairo_paint_with_alpha (
-                            cr, 1.0 * img->trans_step/img->trans_steps);
-                    cairo_restore (cr);
-                } else
-                    cairo_fill (cr);
-            }
+    SRect rect = ir->intrinsicBounds ();
+    if (id && !id->isEmpty () &&
+            img->keepContent (img) &&
+            rect.width() > 0 &&
+            id->width() > 0 && id->height() > 0) {
+        img->cached_rect = rect;
+        Single x = rect.x ();
+        Single y = rect.y ();
+        Single w = rect.width();
+        Single h = rect.height();
+        matrix.getXYWH (x, y, w, h);
+        SRect clip_rect = clip.intersect (SRect (x, y, w, h));
+        if (!clip_rect.isValid ())
+            return;
+        cairo_pattern_t * pat = id->cairoImage (w, h, cairo_surface);
+        if (pat) {
+            cairo_matrix_t mat;
+            cairo_matrix_init_identity (&mat);
+            cairo_matrix_translate (&mat, -x, -y);
+            cairo_pattern_set_matrix (pat, &mat);
+            cairo_pattern_set_filter (pat, CAIRO_FILTER_FAST);
+            cairo_set_source (cr, pat);
+            cairo_rectangle (cr, clip_rect.x (), clip_rect.y (),
+                    clip_rect.width (), clip_rect.height ());
+            if (img->trans_steps > img->trans_step) { // only fadein for now
+                cairo_save (cr);
+                cairo_clip (cr);
+                cairo_paint_with_alpha (
+                        cr, 1.0 * img->trans_step/img->trans_steps);
+                cairo_restore (cr);
+            } else
+                cairo_fill (cr);
         }
     }
 }
@@ -1160,8 +1070,8 @@ KDE_NO_EXPORT void ViewArea::syncVisual (const SRect & rect) {
     int ey = rect.y ();
     if (ey > 0)
         ey--;
-    int ew = rect.width () + 2;
-    int eh = rect.height () + 2;
+    int ew = rect.width () + 3;
+    int eh = rect.height () + 3;
     if (!cairo_surface)
         cairo_surface = cairoCreateSurface (widget, width (), height ());
     CairoPaintVisitor visitor (cairo_surface, SRect (ex, ey, ew, eh));
@@ -1185,10 +1095,6 @@ KDE_NO_EXPORT void ViewArea::resizeEvent () {
     scheduleRepaint (0, 0, w, h);
     if (cairo_surface)
         cairo_xlib_surface_set_size (cairo_surface, w, h);
-#ifdef USE_CAIRO_GLITZ
-    if (glitz_drawable)
-        glitz_drawable_update_size (glitz_drawable, w, h);
-#endif
     // now scale the regions and check if video region is already sized
     bool av_geometry_changed = false;
     surface->bounds = SRect (x, y, w, h);
@@ -1281,8 +1187,17 @@ void ViewArea::moveRect (Single x, Single y, Single w, Single h, Single x1, Sing
 }
 
 KDE_NO_EXPORT void ViewArea::timeoutPaintEvent () {
+    static Single zero;
     m_repaint_timer = 0;
-    syncVisual (m_repaint_rect);
+    if (m_repaint_rect.width () > zero && m_repaint_rect.height () > zero) {
+        syncVisual (m_repaint_rect);
+        if (!m_repaint_timer) {
+            m_repaint_timer = g_timeout_add (40, cb_scheduledPaint, this);
+            m_repaint_rect = SRect ();
+        } else {
+            warningLog () << "scheduleRepaint during syncVisual" << endl;
+        }
+    }
 }
 
 KDE_NO_EXPORT int ViewArea::width () const {
